source("../../other/asmisc.r")
library(MASS)
library(cluster)
library(tree)

# ===================================
# Funkcii perekrytija, grafiki

perekr <- function (x, y) {
if ((x[2] > y[2]) | ((x[2] == y[2]) & (x[1] <= y[1])))
	{
	pr <- x
	le <- y
	mesto=0
  }
else
	{
	pr <- y
	le <- x
	mesto=1
  }
if (pr[1] >= le[2]) return(0)
else
	{
	if ((pr[2] >= le[2]) & (le[1] >= pr[1]))
	  {
		if (mesto == 0) return((le[2]-le[1])/(pr[2]-pr[1]))
		else return(1)
	  }
	else return((le[2]-pr[1])/(pr[2]-le[1]))
	}
return(res)
}

myrange <- function(a, num=0) {
b <- as.numeric(names(table(a)))
if ((num+1) <= (length(b)-num)) d <- (num+1):(length(b)-num)
else d <- 0
b <- b[d]
a <- a[a %in% b]
return (range(a))
}

taper <- function (priznak, sp, okr=2)
{
prnew <- na.omit(data.frame(sp=sp, priznak=priznak))
vidy <- names(table(prnew$sp))
len <- length(vidy)
mat <- matrix(ncol=len, nrow=len)
for (x in 1:len)
{
  for (y in 1:len)
  {
    xx <- myrange(prnew$priznak[prnew$sp==vidy[x]])
    yy <- myrange(prnew$priznak[prnew$sp==vidy[y]])
    mat[x,y] <- round (perekr (xx, yy), okr)
  }
}
dimnames(mat) <- list(vidy, vidy)
return(mat)
}

d.plot <- function(priznak, sp, sp.list=sort(unique(sp)),
  nazv=deparse(substitute(priznak)), fnazv=deparse(substitute(priznak)),
	ht=.01, w=8, h=6, xlab="", ylab="", ...)
{
prnew <- data.frame(sp=sp, priznak=priznak)
pdf(file=paste(fnazv, ".pdf", sep=""), width=w, height=h)
plot(density(prnew$priznak, na.rm=TRUE), xlim=range(prnew$priznak, na.rm=TRUE),
	yaxt="n", type="n", ylim=c(0,ht), main=nazv, xlab=xlab, ylab=ylab, ...)
for (a in 1:length(sp.list)) {
	lines(density(prnew$priznak[sp==sp.list[a]], na.rm=TRUE), lty=a)
  }
legend(0.8*max(prnew$priznak, na.rm=TRUE), 0.8*ht, sp.list, lty=1:length(sp.list))
dev.off()
}

# ===================================
# ===================================
# ===================================

eq <- read.table("../data/eq-s.dat", h=T, sep=";")
eql <- read.table("../data/eq-l.dat", h=T, sep=";")

eq$VID <- recode (eq$N.POP, eql$N.POP, eql$SPECIES)
sp <- recode(eq$N.POP, eql$N.POP, as.character(eql$SPECIES))
sp.u <- c("arvense","fluviatile","hyemale","palustre","pratense","sylvaticum")
pr.u <- c("DL.R","DIA.ST","DIA.POL","N.REB","N.ZUB","DL.OSN.Z","DL.TR.V", "DL.BAZ","DL.PER","N.UG.PER")

# ======

for (a in 1:length(pr.u)){
	cat("\n================\n",pr.u[a],"\n")
	print(taper(eq[,pr.u[a]], sp))}

d.plot(eq$DL.R, sp, sp.u, ht=.005, nazv="Plant length")
d.plot(eq$DIA.ST, sp, sp.u, ht=1.5, nazv="Stem diameter")

otn <- eq$DL.BAZ/eq$DL.TR.V
d.plot(otn, sp, sp.u, ht=2.5, nazv="Basal internode / sheath length")

# ======

eq1 <- eq[,-c(12, 21)]
eq1 <- na.omit(eq1)
eq1.d <- daisy(scale(eq1[,c(2:19)]), metric="manhattan")
eq1.i <- isoMDS(eq1.d)
col <- recode(eq1$VID, c(1:8), c("black", "blue", "violet", "coral", "red", "green", "gray", "brown"))
eqscplot(eq1.i$points, type="n", main="MDS, equisetum, species")
text(eq1.i$points, labels= eq1[,1], col=col, cex=.8)
legend (12, 20, c("arv", "flu", "hye", "lit", "mil", "pal", "pra", "syl"), fill=c("black", "blue", "violet", "coral", "red", "green", "gray", "brown"))

# ======

eq$VID <- as.factor(eq$VID)
eq.t <- (tree(VID ~ DL.R + VYS.UZ + N.UZ + VET + DIA.ST + DIA.POL + N.REB + SHIP + N.ZUB + SRAS + DL.OSN.Z + DL.OST.Z + DL.TR.V + DL.BAZ + DL.PER + TR.PER.Z + N.UG.PER + MRSH, data= eq))
plot(eq.t)
text (eq.t, cex=.8)

# ===================================
# klassifikatsiya po Skvortsovu

eq.veg <- eq
eq.veg$SP <- sp
eq.veg$KOLO[is.na(eq.veg$KOLO)] <- 0
eq.veg <- eq.veg[eq.veg$KOLO == 0 & eq.veg$SP != "hyemale",]
SKV <- rep(0, nrow(eq.veg))
attach(eq.veg)

SKV[(SKV == "0") & (VET > 0 & SHIP == 1 & MRSH == 0 & VET == 2 & N.ZUB > 3 & N.ZUB < 12)] <- "sylvaticum"

SKV[(SKV == "0") & ((SHIP == 1) | (VET > 0 & MRSH == 0 & VET == 1 & DL.PER/(DL.TR.V + DL.OSN.Z + DL.OST.Z) > 1)) ] <- "pratense"

SKV[(DIA.ST > 4 & N.ZUB > 15 & DL.R > 500) | (VET < 2 & SHIP == 0 & SHOV == 0 & VET <= 1 & DIA.POL/DIA.ST > 5/6 & DIA.ST > 2 & N.ZUB > 6 & DL.R > 200)] <- "fluviatile"

SKV[(SKV == "0") & ((VET < 2 & SHIP == 0 & SHOV == 1 & DIA.POL/DIA.ST <= 2/3 & (DL.TR.V + DL.OSN.Z + DL.OST.Z)/DL.PER > 1.5 & DL.PER/DL.BAZ <= 3) | (MRSH == 1))] <- "palustre" 

SKV[(SKV == "0") & (VET < 2 & SHIP == 0 & SHOV == 1 & DIA.POL/DIA.ST <= 2/3 & DL.PER/(DL.TR.V + DL.OSN.Z + DL.OST.Z) > 1 & DL.PER/DL.BAZ > 3 & MRSH == 0)] <- "arvense"
detach(eq.veg)
table(eq.veg$SP, SKV)

# Nado: klassifikacija po "Flora Nordica" i po "Plant Crib"